﻿namespace LarsW.WcfTestClientEx.TestService
{
    using System;
    using System.ServiceModel;
    using System.Runtime.Serialization;
    using System.ServiceModel.Description;

    class Program
    {
        static void Main()
        {
            var host = new ServiceHost(typeof (Service), new Uri("http://localhost:12345/"));
            
            var smdb = new ServiceMetadataBehavior { HttpGetEnabled = true };
            host.Description.Behaviors.Add(smdb);

            host.AddServiceEndpoint(typeof(IService), new BasicHttpBinding(BasicHttpSecurityMode.None), string.Empty);
            host.AddServiceEndpoint(typeof (IMetadataExchange), MetadataExchangeBindings.CreateMexHttpBinding(), "mex");
            host.Open();
            Console.WriteLine("Service is running. Press enter to shutdown.");
            Console.ReadLine();
            host.Close();
        }
    }

    [ServiceContract]
    interface IService
    {
        [OperationContract]
        string SimpleMethod(string input);
        
        [OperationContract]
        Response ComplexMethod(Request request);
    }

    class Service : IService
    {
        #region Implementation of IService

        public string SimpleMethod(string input)
        {
            return string.Format("Hello {0}", input);
        }

        public Response ComplexMethod(Request request)
        {
            return new Response {Value = string.Format("Text: {0}, Integer: {1}", request.Text, request.Integer)};
        }

        #endregion
    }

    [DataContract]
    class Request
    {
        [DataMember]
        public string Text { get; set; }
        [DataMember]
        public int Integer { get; set; }
    }

    [DataContract]
    class Response
    {
        [DataMember]
        public string Value { get; set; }
    }
}
