﻿namespace LarsW.WcfTestClientEx
{
    using System;
    using System.Diagnostics;
    using System.Globalization;
    using System.IO;
    using System.Reflection;
    using System.Windows.Forms;
    using System.Runtime.Serialization;

    internal static class Program
    {
        private static Form _form;

        private const BindingFlags NonPublicInstanceFieldBindingFlags = BindingFlags.NonPublic | BindingFlags.GetField | BindingFlags.Instance;

        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        private static void Main(string[] args)
        {
            AppDomain.CurrentDomain.TypeResolve += GracefullyHandleResolveFailure;
            AppDomain.CurrentDomain.AssemblyResolve += GracefullyHandleResolveFailure;
            Application.EnableVisualStyles();
            Application.SetCompatibleTextRenderingDefault(false);

            if (args.Length == 0)
            {
                ShowServiceMetadataRetrievalError();
            }
            else
            {
                foreach (string str in args)
                {
                    if (string.Equals("/?", str, StringComparison.Ordinal) ||
                        string.Equals("-?", str, StringComparison.Ordinal))
                    {
                        // ShowUsage();
                        return;
                    }
                }

                var testClientAssembly = Assembly.LoadFrom("WcfTestClient.exe");
                var mainFormType = testClientAssembly.GetType("Microsoft.Tools.TestClient.UI.MainForm");
                var servicePageType = testClientAssembly.GetType("Microsoft.Tools.TestClient.UI.ServicePage");
                var serviceMethodInfoType = testClientAssembly.GetType("Microsoft.Tools.TestClient.ServiceMethodInfo");
                var testCaseType = testClientAssembly.GetType("Microsoft.Tools.TestClient.TestCase");
                var formattedPageType = testClientAssembly.GetType("Microsoft.Tools.TestClient.UI.FormattedPage");
                var variableInfoType = testClientAssembly.GetType("Microsoft.Tools.TestClient.VariableInfo");

                var tcFi = servicePageType.GetField("testCase", NonPublicInstanceFieldBindingFlags);
                var smiFi = testCaseType.GetField("method", NonPublicInstanceFieldBindingFlags);
                var nameFi = serviceMethodInfoType.GetField("methodName", NonPublicInstanceFieldBindingFlags);
                var paramFi = serviceMethodInfoType.GetField("parameters", NonPublicInstanceFieldBindingFlags);
                var icFi = formattedPageType.GetField("inputControl", NonPublicInstanceFieldBindingFlags);

                var fpFi = servicePageType.GetField("formattedPage", NonPublicInstanceFieldBindingFlags);

                var ci = mainFormType.GetConstructor(BindingFlags.Instance | BindingFlags.NonPublic, null,
                                                                 new[] {typeof (string[])}, null);

                _form = ci.Invoke(new object[] {new[] {args[0]}}) as Form;
                // TODO - if _form == null - exit.
                _form.Text = _form.Text + " Extended!";


                var saveButton = new ToolStripButton("&Save test data...");
                saveButton.Click += delegate
                                        {
                                            var fi = mainFormType.GetField("serviceTabControl", NonPublicInstanceFieldBindingFlags);
                                            var serviceTabControl = fi.GetValue(_form) as TabControl;
                                            if (!serviceTabControl.SelectedTab.GetType().Equals(servicePageType))
                                                return;

                                            var formattedPage = fpFi.GetValue(serviceTabControl.SelectedTab);

                                            var getVariablesMi = formattedPageType.GetMethod("GetVariables",
                                                                                                    BindingFlags.
                                                                                                        Instance |
                                                                                                    BindingFlags.
                                                                                                        NonPublic);
                                            object variableInfos;
                                            try
                                            {
                                                variableInfos = getVariablesMi.Invoke(formattedPage,
                                                                                      BindingFlags.NonPublic |
                                                                                      BindingFlags.Instance |
                                                                                      BindingFlags.InvokeMethod,
                                                                                      null, null,
                                                                                      CultureInfo.CurrentCulture);
                                            }
                                            catch (Exception ex)
                                            {
                                                Debug.WriteLine(ex.ToString());
                                                throw;
                                            }

                                            var serializer = new NetDataContractSerializer();
                                            using (var sfd = new SaveFileDialog())
                                            {
                                                sfd.FileName = "parameters.testdata";
                                                if (sfd.ShowDialog() == DialogResult.OK)
                                                {
                                                    sfd.Filter = "Test data (*.testdata)|*.testdata|All files (*.*)|*.*";
                                                    using (var fs = File.Create(sfd.FileName))
                                                    {
                                                        serializer.Serialize(fs, variableInfos);
                                                    }
                                                }
                                            }

                                            // TODO set global status bar message.
                                        };

                var loadButton = new ToolStripButton("&Load test data...");
                loadButton.Click += delegate
                                        {
                                            using (var ofd = new OpenFileDialog())
                                            {
                                                ofd.Filter = "Test data (*.testdata)|*.testdata|All files (*.*)|*.*";
                                                if (ofd.ShowDialog() == DialogResult.Cancel)
                                                    return;

                                                var serializer = new NetDataContractSerializer();
                                                using (var  fs = File.Open(ofd.FileName, FileMode.Open))
                                                {
                                                    var parameters = serializer.Deserialize(fs);

                                                    var fi = mainFormType.GetField("serviceTabControl", NonPublicInstanceFieldBindingFlags);
                                                    var serviceTabControl = fi.GetValue(_form) as TabControl;
                                                    if (!serviceTabControl.SelectedTab.GetType().Equals(servicePageType))
                                                        return;

                                                    var formattedPage = fpFi.GetValue(serviceTabControl.SelectedTab);

                                                    var populateTreeMi =
                                                        formattedPageType.GetMethod("PopulateTree",
                                                                                    BindingFlags.Instance |
                                                                                    BindingFlags.NonPublic |
                                                                                    BindingFlags.InvokeMethod);

                                                    var inputControl = icFi.GetValue(formattedPage);

                                                    try
                                                    {
                                                        populateTreeMi.Invoke(formattedPage,
                                                                              BindingFlags.InvokeMethod |
                                                                              BindingFlags.NonPublic |
                                                                              BindingFlags.Instance, null,
                                                                              new[] {parameters, inputControl, false},
                                                                              CultureInfo.CurrentCulture);
                                                    }
                                                    catch (Exception ex)
                                                    {
                                                        Debug.WriteLine(ex.ToString());
                                                        throw;
                                                    }
                                                }
                                            }
                                        };
                var fileItem = _form.MainMenuStrip.Items[0] as ToolStripMenuItem;
                fileItem.DropDownItems.Insert(0, saveButton);
                fileItem.DropDownItems.Insert(0, loadButton);

                var aboutItem = new ToolStripButton("About the load/save extension...", null, delegate
                                                                                                  {
                                                                                                      using (
                                                                                                          var form =
                                                                                                              new ExtensionAboutForm
                                                                                                                  ())
                                                                                                      {
                                                                                                          form.
                                                                                                              ShowDialog
                                                                                                              ();
                                                                                                      }
                                                                                                  });

                ((ToolStripMenuItem) _form.MainMenuStrip.Items[1]).DropDownItems.Add(aboutItem);
                Application.Run(_form);
            }
        }

        private static Assembly GracefullyHandleResolveFailure(object sender, ResolveEventArgs args)
        {
            //MessageBox.Show(string.Format(CultureInfo.CurrentUICulture, StringResources.DependencyResolveFailure, new object[] { args.Name }), StringResources.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Hand, MessageBoxDefaultButton.Button1, 0);
            Application.Exit();
            return null;
        }


        private static void ShowServiceMetadataRetrievalError()
        {
            // TODO MessageBox.Show(StringResources.ErrorServiceMetadataCannotBeRetrieved, StringResources.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button1, 0);
        }
    }
}